package flags

import "strconv"

type BoolFlag struct {
	name        string
	shortName   string
	description string
	value       bool
}

var _ Flag = (*StringFlag)(nil)

func Bool(name, shortName string, defaultValue bool, description string) *BoolFlag {
	return &BoolFlag{
		name:        name,
		shortName:   shortName,
		description: description,
		value:       defaultValue,
	}
}

// Description implements Flag.
func (f *BoolFlag) Description() string {
	return f.description
}

// Name implements Flag.
func (f *BoolFlag) Name() string {
	return f.name
}

// ShortName implements Flag.
func (f *BoolFlag) ShortName() string {
	return f.shortName

}

// Value implements Flag.
func (f *BoolFlag) Parse(raw string) error {
	if raw == "" {
		return nil
	}

	boolVal, err := strconv.ParseBool(raw)
	if err != nil {
		return err
	}

	f.value = boolVal
	return nil
}

func (f *BoolFlag) Value() bool {
	return f.value
}
