package flags

import "strconv"

type IntFlag struct {
	name        string
	shortName   string
	description string
	value       int
}

var _ Flag = (*StringFlag)(nil)

func Int(name, shortName string, defaultValue int, description string) *IntFlag {
	return &IntFlag{
		name:        name,
		shortName:   shortName,
		description: description,
		value:       defaultValue,
	}
}

// Description implements Flag.
func (f *IntFlag) Description() string {
	return f.description
}

// Name implements Flag.
func (f *IntFlag) Name() string {
	return f.name
}

// ShortName implements Flag.
func (f *IntFlag) ShortName() string {
	return f.shortName

}

// Value implements Flag.
func (f *IntFlag) Parse(raw string) error {
	if raw == "" {
		return nil
	}

	intVal, err := strconv.Atoi(raw)
	if err != nil {
		return err
	}

	f.value = intVal
	return nil
}

func (f *IntFlag) Value() int {
	return f.value
}
